# services connectability

Here is an overview of the connectability ("connectable" or "not connectable" or "in the plan" or "never") to some services by this plugin.

The plugin works under the browser environment in Obsidian, so CORS is an issue. Obsidian starts to provide a rich API `requestUrl` for desktop version >= 0.13.25, mobile >= 1.1.1 to bypass the CORS issue. But if the users are still using an older version of Obsidian, they need to configure CORS on server.

The list is for information purposes only.

| Service                                                                               | Connectable | by S3 | by WebDAV | by other protocol                                                | can bypass CORS issue in latest Obsidian | need CORS config in old Obsidian                 |
| ------------------------------------------------------------------------------------- | ----------- | ----- | --------- | ---------------------------------------------------------------- | ---------------------------------------- | ------------------------------------------------ |
| Amazon S3                                                                             | Yes         | Yes   |           |                                                                  | Yes                                      | [CORS config needed.](./s3_cors_configure.md)    |
| Tencent Cloud - Cloud Object Storage (COS) 腾讯云对象存储                             | Yes         | Yes   |           |                                                                  | Yes                                      | CORS config needed.                              |
| Alibaba Cloud - Object Storage Service 阿里云对象存储                                 | Yes         | Yes   |           |                                                                  | Yes                                      | CORS config needed.                              |
| Backblaze B2 Cloud Storage                                                            | Yes         | Yes   |           |                                                                  | Yes                                      | Its CORS rules doesn't allow no-http(s) origins. |
| [Wasabi](https://wasabi.com)                                                          | ?           | ?     |           |                                                                  |                                          |                                                  |
| [filebase](https://filebase.com/)                                                     | Yes         | Yes   |           |                                                                  | Yes                                      | CORS config needed.                              |
| QingStor 青云                                                                         | ?           | ?     |           |                                                                  |                                          |                                                  |
| [MinIO](https://min.io/)                                                              | ?           | ?     |           |                                                                  |                                          |                                                  |
| [WsgiDAV](https://github.com/mar10/wsgidav)                                           | Yes         |       | Yes       |                                                                  | Yes                                      | CORS rules can be set.                           |
| [Nginx `ngx_http_dav_module`](http://nginx.org/en/docs/http/ngx_http_dav_module.html) | Yes?        |       | Yes?      |                                                                  | Yes?                                     | ?                                                |
| NextCloud                                                                             | Yes         |       | Yes       |                                                                  | Yes?                                     | No CORS config by default.                       |
| OwnCloud                                                                              | Yes?        |       | Yes?      |                                                                  | Yes?                                     | No CORS config by default.                       |
| Seafile                                                                               | Yes         |       | Yes       |                                                                  | Yes?                                     | No CORS config by default.                       |
| `rclone serve webdav`                                                                 | Yes         |       | Yes       |                                                                  | Yes                                      | No CORS support.                                 |
| [Nutstore 坚果云](https://www.jianguoyun.com/)                                        | Yes         |       | Yes       |                                                                  | Yes                                      | No CORS support.                                 |
| [TeraCLOUD](https://teracloud.jp/en/)                                                 | Yes         |       | Yes       |                                                                  | Yes                                      | No CORS support.                                 |
| Dropbox                                                                               | Yes         |       |           | Yes                                                              |                                          |                                                  |
| OneDrive for personal                                                                 | Yes         |       |           | Yes                                                              |                                          |                                                  |
| OneDrive for Business                                                                 | Yes         |       |           | ?                                                                |                                          |                                                  |
| Google Drive                                                                          | In the plan |       |           | ?                                                                |                                          |                                                  |
| [Box](https://www.box.com/)                                                           | ?           |       |           | May be possible but needs further development.                   |                                          |                                                  |
| Google Cloud Storage                                                                  | ?           |       |           | May be possible but needs further development.                   |                                          |                                                  |
| Microsoft Azure Blob Storage                                                          | ?           |       |           | May be possible but needs further development.                   |                                          |                                                  |
| [OpenStack Storage (Swift)](https://github.com/openstack/swift)                       | ?           |       |           | May be possible but needs further development.                   |                                          |                                                  |
| https://put.io/                                                                       | ?           |       |           | ?                                                                |                                          |                                                  |
| Yandex Disk                                                                           | ?           |       |           | ?                                                                |                                          |                                                  |
| FTP / FTPS                                                                            | Never       |       |           | Technically never possible to be implemented.                    |                                          |                                                  |
| SFTP                                                                                  | Never       |       |           | Technically never possible to be implemented.                    |                                          |                                                  |
| Jottacloud                                                                            | No          |       |           | No. It seems that no open api is available.                      |                                          |                                                  |
| Mega                                                                                  | Never       |       |           | No. No js api is available.                                      |                                          |                                                  |
| Git                                                                                   | Never       |       |           | No. Technically very hard, if not impossible, to be implemented. |                                          |                                                  |
|                                                                                       |             |       |           |                                                                  |                                          |                                                  |
